/*
 * Copyright 2014 Kevin Funk <kfunk@kde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License or (at your option) version 3 or any later version
 * accepted by the membership of KDE e.V. (or its successor approved
 * by the membership of KDE e.V.), which shall act as a proxy
 * defined in Section 14 of version 3 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef CLANGDEBUG_H
#define CLANGDEBUG_H

#include <QLoggingCategory>

#include <clang-c/Index.h>

#include "clangprivateexport.h"

extern KDEVCLANGPRIVATE_EXPORT const QLoggingCategory &KDEV_CLANG();
#define clangDebug() qCDebug(KDEV_CLANG)

KDEVCLANGPRIVATE_EXPORT QDebug operator<<(QDebug dbg, CXString string);
KDEVCLANGPRIVATE_EXPORT QDebug operator<<(QDebug dbg, CXSourceLocation location);
KDEVCLANGPRIVATE_EXPORT QDebug operator<<(QDebug dbg, CXSourceRange range);
KDEVCLANGPRIVATE_EXPORT QDebug operator<<(QDebug dbg, CXCursor cursor);
KDEVCLANGPRIVATE_EXPORT QDebug operator<<(QDebug dbg, CXCursorKind kind);
KDEVCLANGPRIVATE_EXPORT QDebug operator<<(QDebug dbg, CXType type);
KDEVCLANGPRIVATE_EXPORT QDebug operator<<(QDebug dbg, CXTypeKind typeKind);

#endif // CLANGDEBUG_H
