/* This file is part of KDevelop
 *
 * Copyright 2018 Amish K. Naidu <amhndu@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#ifndef SCRATCHPADVIEW_H
#define SCRATCHPADVIEW_H

#include <QWidget>

#include <vector>

#include "ui_scratchpadview.h"

class Scratchpad;

class QModelIndex;
class QLineEdit;
class QAbstractProxyModel;

class ScratchpadView
    : public QWidget
    , public Ui::ScratchpadBaseView
{
    Q_OBJECT

public:
    ScratchpadView(QWidget* parent, Scratchpad* scratchpad);

private Q_SLOTS:
    void runSelectedScratch();
    void scratchActivated(const QModelIndex& index);
    void createScratch();

private:
    QAbstractProxyModel* proxyModel() const;
    QModelIndex currentIndex() const;

    void setupActions();
    void validateItemActions();

    Scratchpad* m_scratchpad;
    QLineEdit* m_filter = nullptr;
    QVector<QAction*> m_itemActions;
};

#endif // SCRATCHPADVIEW_H
