/* This file is part of KDevelop

   Copyright 2018 Anton Anikin <anton@anikin.xyz>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#ifndef KDEVCLAZY_COMMAND_LINE_WIDGET_H
#define KDEVCLAZY_COMMAND_LINE_WIDGET_H

#include <QWidget>

namespace Clazy
{

namespace Ui { class CommandLineWidget; }

class CommandLineWidget : public QWidget
{
    Q_OBJECT

public:
    explicit CommandLineWidget(QWidget* parent = nullptr);
    ~CommandLineWidget() override;

public Q_SLOTS:
    void setText(const QString& text);

private:
    void updateCommandLine();

private:
    QScopedPointer<Ui::CommandLineWidget> m_ui;
    QString m_text;
};

}

#endif
