/* KDevelop GDB Support
 *
 * Copyright 2009 Aleix Pol <aleixpol@kde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#ifndef PROCESSSELECTION_H
#define PROCESSSELECTION_H

#include <QDialog>

class KSysGuardProcessList;
class QItemSelection;
class QPushButton;

namespace KDevMI {

class ProcessSelectionDialog : public QDialog
{
    Q_OBJECT
    public:
        explicit ProcessSelectionDialog( QWidget *parent=nullptr );
        ~ProcessSelectionDialog() override;
        long int pidSelected();
        QSize sizeHint() const override;

    private Q_SLOTS:
        void selectionChanged( const QItemSelection& selected );

    private:
        KSysGuardProcessList* m_processList;
        QPushButton* m_attachButton;
};

} // end of namespace KDevMI

#endif
