<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="KDevFileTemplates" version="2" translationDomain="kdevfiletemplates">
  <MenuBar>
    <Menu name="file">
      <Action name="new_from_template" group="new_merge"/>
    </Menu>
  </MenuBar>
</gui>
