/*
    This plugin is part of KDevelop.

    Copyright (C) 2010 Milian Wolff <mail@milianw.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

 */

#ifndef KDEVPLATFORM_PLUGIN_EDITEXTERNALSCRIPT_H
#define KDEVPLATFORM_PLUGIN_EDITEXTERNALSCRIPT_H

#include <QDialog>

#include "ui_editexternalscript.h"

class ExternalScriptItem;

class EditExternalScript
    : public QDialog
    , private Ui::EditExternalScriptBase
{
    Q_OBJECT

public:
    explicit EditExternalScript(ExternalScriptItem* item, QWidget* parent = nullptr);
    ~EditExternalScript() override;

private Q_SLOTS:
    void save();
    void validate();

private:
    ExternalScriptItem* m_item;
};

#endif // KDEVPLATFORM_PLUGIN_EDITEXTERNALSCRIPT_H
