/*
 * This file is part of KDevelop
 *
 * Copyright 2007-2008 David Nolden <david.nolden.kdevelop@art-master.de>
 * Copyright 2006 Hamish Rodda <rodda@kde.org>
 * Copyright 2009 Milian Wolff <mail@milianw.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#ifndef KDEVPLATFORM_CONFIGURABLECOLORS_H
#define KDEVPLATFORM_CONFIGURABLECOLORS_H

#include <language/languageexport.h>

#include <KTextEditor/Attribute>

namespace KDevelop {
class ColorCache;

class KDEVPLATFORMLANGUAGE_EXPORT ConfigurableHighlightingColors
{
public:
    explicit ConfigurableHighlightingColors();

    void addAttribute(int number, const KTextEditor::Attribute::Ptr& attribute);

    KTextEditor::Attribute::Ptr attribute(int number) const;

    void setDefaultAttribute(const KTextEditor::Attribute::Ptr& defaultAttrib);

    KTextEditor::Attribute::Ptr defaultAttribute() const;

private:
    KTextEditor::Attribute::Ptr m_defaultAttribute;
    QHash<int, KTextEditor::Attribute::Ptr> m_attributes;
    QString m_highlightingName;
};

// ######### start CodeHighlightingColors

class CodeHighlightingColors
    : public ConfigurableHighlightingColors
{
public:
    explicit CodeHighlightingColors(ColorCache* cache);
};
}

#endif // KDEVPLATFORM_CONFIGURABLECOLORS_H
