/*
 * This file is part of KDevelop
 *
 * Copyright 2018 Friedrich W. H. Kossebau <kossebau@kde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#ifndef CLANGTIDY_CHECKSET_H
#define CLANGTIDY_CHECKSET_H

// Qt
#include <QStringList>

namespace ClangTidy
{

/**
 * \brief Provides all available checks by running clang-tidy with the following parameters: "--checks=*
 * --list-checks".
 */
class CheckSet
{
public:
    CheckSet() = default;

public:
    /**
     * @param path the system path for the clang-tidy program.
     */
    void setClangTidyPath(const QString& path);

    const QStringList &all() const { return m_allChecks; }
    QStringList defaults() const;

private:
    QString m_clangTidyPath;
    QStringList m_allChecks;
};

}

#endif
