/*
 * This file is part of qmljs, the QML/JS language support plugin for KDevelop
 * Copyright (c) 2014 Denis Steckelmacher <steckdenis@yahoo.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License or (at your option) version 3 or any later version
 * accepted by the membership of KDE e.V. (or its successor approved
 * by the membership of KDE e.V.), which shall act as a proxy
 * defined in Section 14 of version 3 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
    
exports.log = function (data) { return ; };
exports.log(_mixed);

exports.info = function (data) { return ; };
exports.info(_mixed);

exports.error = function (data) { return ; };
exports.error(_mixed);

exports.warn = function (data) { return ; };
exports.warn(_mixed);

exports.dir = function (obj) { return ; };
exports.dir(new Object());

exports.time = function (label) { return ; };
exports.time("");

exports.timeEnd = function (label) { return ; };
exports.timeEnd("");

exports.trace = function (label) { return ; };
exports.trace("");

exports.assert = function (expression, message) { return ; };
exports.assert(true, "");

