/* This file is part of KDevelop
   Copyright 2013 Milian Wolff <mail@milianw.de>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "testsuite.h"

#include <language/duchain/declaration.h>

using namespace KDevelop;

TestSuite<Declaration*>& KDevelop::declarationTestSuite()
{
    static TestSuite<Declaration*> suite;
    return suite;
}

TestSuite<DUContext*>& KDevelop::contextTestSuite()
{
    static TestSuite<DUContext*> suite;
    return suite;
}

TestSuite<AbstractType::Ptr>& KDevelop::typeTestSuite()
{
    static TestSuite<AbstractType::Ptr> suite;
    return suite;
}
