/***************************************************************************
 *   Copyright 2008 David Nolden <david.nolden.kdevelop@art-master.de>     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
 ***************************************************************************/

#ifndef KDEVPLATFORM_COMPLETIONSETTINGS_H
#define KDEVPLATFORM_COMPLETIONSETTINGS_H

#include <interfaces/icompletionsettings.h>
#include <KConfigGroup>

namespace KDevelop
{

class CompletionSettings : public KDevelop::ICompletionSettings
{
    Q_OBJECT
public:
    CompletionLevel completionLevel() const override;

    bool automaticCompletionEnabled() const override;

    void emitChanged() { emit settingsChanged(this); }

    int localColorizationLevel() const override;

    int globalColorizationLevel() const override;

    bool highlightSemanticProblems() const override;

    bool highlightProblematicLines() const override;

    bool boldDeclarations() const override;

    bool showMultiLineSelectionInformation() const override;

    int minFilesForSimplifiedParsing() const override;

    QStringList todoMarkerWords() const override;

    static CompletionSettings& self();

private:
    CompletionSettings();

    const CompletionLevel m_level = MinimalWhenAutomatic;
    const bool m_automatic = true;
    const bool m_highlightSemanticProblems = true;
    const bool m_highlightProblematicLines = false;
    const bool m_showMultiLineInformation = false;
    const bool m_boldDeclarations = true;
    const int m_localColorizationLevel = 170;
    const int m_globalColorizationLevel = 255;
    const int m_minFilesForSimplifiedParsing = 100000;
    const QString m_todoMarkerWords;

    const KConfigGroup m_languageGroup;
};
}
#endif
