/* KDevelop CMake Support
 *
 * Copyright 2016 René J.V. Bertin <rjvbertin@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#ifndef CMAKEEXTRAARGUMENTS_H
#define CMAKEEXTRAARGUMENTS_H

#include <KComboBox>

#include "cmakecommonexport.h"

class QStringList;

namespace Ui {
    class CMakeExtraArgumentsHistory;
}

/**
* This class augments CMake's extra arguments widget with a history feature.
* @author René Bertin
*/
class KDEVCMAKECOMMON_EXPORT CMakeExtraArgumentsHistory
{
    public:
        explicit CMakeExtraArgumentsHistory(KComboBox* widget);
        ~CMakeExtraArgumentsHistory();

        QStringList list() const;

    private:
        Q_DISABLE_COPY(CMakeExtraArgumentsHistory)

        KComboBox* m_arguments;
};

#endif
