/*
 *    This file is part of KDevelop
 *
 *    Copyright 2013 Olivier de Gaalon <olivier.jg@gmail.com>
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Library General Public
 *    License as published by the Free Software Foundation; either
 *    version 2 of the License, or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public License
 *    along with this library; see the file COPYING.LIB.  If not, write to
 *    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *    Boston, MA 02110-1301, USA.
 */

#ifndef TEMPLATEHELPERS_H
#define TEMPLATEHELPERS_H

#include <QDebug>

#include <type_traits>

enum class Decision
{
    True,
    False,
    Maybe
};

// see also: http://flamingdangerzone.com/cxx11/2012/06/01/almost-static-if.html
namespace detail { enum class enabler {}; }
constexpr detail::enabler dummy = {};
template <bool Condition>
using EnableIf = typename std::enable_if<Condition, detail::enabler>::type;

inline QDebug operator<<(QDebug dbg, Decision decision)
{
    switch (decision) {
    case Decision::True:  dbg << "true"; break;
    case Decision::False: dbg << "false"; break;
    default:              dbg << "maybe"; break;
    }
    return dbg;
}

#endif //TEMPLATEHELPERS_H
